#!/usr/bin/env python3
# _*_ coding: utf-8 _*_

__author__ = 'ChenzDNA'

from setuptools import setup, find_packages

with open("README.md", "r", encoding='utf-8') as fh:
    long_description = fh.read()

setup(
    name="pypadb",
    version="0.1.9",
    description='Interaction with database in a simple way',
    author="ChenzDNA",
    author_email='1007324849@qq.com',
    url='https://github.com/ChenzDNA/pypa',
    packages=find_packages(),
    long_description=long_description,
    long_description_content_type="text/markdown",
    license="Apache-2.0",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent"],
    python_requires='>=3.5',
    install_requires=[
        "DBUtils>=2.0.2",
        "pydantic>=1.8.2",
        "PyMySQL>=1.0.2",
    ]
)



import functools

from pypadb.utils import inspect_util
from pypadb.utils.query_util import execute


def delete(sql: str):
    def deco(fun):
        @functools.wraps(fun)
        def wrapper(*args):
            fun(*args)
            execute(sql, dict(zip([a.name for a in inspect_util.arg_list(fun)], args)))

        return wrapper

    return deco



import pymysql

from pypadb.connection_pool import _init_pool


class DbConfigurer():
    __data: dict = {
        "creator": pymysql,
        "maxconnections": 6,
        "mincached": 2,
        "maxcached": 5,
        "blocking": True,
        "maxusage": None,
        "setsession": [],
        "ping": 0,
        "host": "127.0.0.1",
        "port": 3306,
        "user": "root",
        "password": "123456",
        "database": "test",
        "charset": "utf8",
        "cursor": pymysql.cursors.DictCursor
    }

    def end(self) -> None:
        _init_pool(**self.__data)

    def set_creator(self, creator) -> 'DbConfigurer':
        self.__data['creator'] = creator
        return self

    def set_maxconnections(self, maxconnections: int) -> 'DbConfigurer':
        self.__data['maxconnections'] = maxconnections
        return self

    def set_mincached(self, mincached: int) -> 'DbConfigurer':
        self.__data['mincached'] = mincached
        return self

    def set_maxcached(self, maxcached: int) -> 'DbConfigurer':
        self.__data['maxcached'] = maxcached
        return self

    def set_blocking(self, blocking: bool) -> 'DbConfigurer':
        self.__data['blocking'] = blocking
        return self

    def set_maxusage(self, maxusage: int) -> 'DbConfigurer':
        self.__data['creator'] = maxusage
        return self

    def set_setsession(self, setsession: list[str]) -> 'DbConfigurer':
        self.__data['setsession'] = setsession
        return self

    def set_host(self, host: str) -> 'DbConfigurer':
        self.__data['host'] = host
        return self

    def set_port(self, port: int) -> 'DbConfigurer':
        self.__data['port'] = port
        return self

    def set_user(self, user: str) -> 'DbConfigurer':
        self.__data['user'] = user
        return self

    def set_password(self, password: str) -> 'DbConfigurer':
        self.__data['password'] = password
        return self

    def set_database(self, database: str) -> 'DbConfigurer':
        self.__data['database'] = database
        return self

    def set_charset(self, charset: str) -> 'DbConfigurer':
        self.__data['charset'] = charset
        return self

    def set_cursor(self, cursor) -> 'DbConfigurer':
        self.__data['cursor'] = cursor
        return self


db_configurer: DbConfigurer = DbConfigurer()



from typing import Any, Union

from ..utils.conditions import Limit, Like
from ..utils.enums import QueryModeEnum
from ..utils.query_util import query, execute, parse_sql_batch, parse_sql_where, parse_sql_update


class BaseTable:
    __slots__ = ['name', 'data_type', 'base_select_sql']

    name: str
    data_type: Any
    base_select_sql: str

    def __init__(self, name: str, data_type):
        self.name = name
        self.data_type = data_type
        self.base_select_sql = f'select * from {name}'

    def select_one(self, extra=None, **kwargs):
        res, _ = query(parse_sql_where(self.base_select_sql, kwargs), kwargs, self.data_type, QueryModeEnum.One)
        return extra(res) if extra else res

    def select_many(self, limit: Limit = None, extra=None, **kwargs) -> list:
        res, _ = query(parse_sql_where(self.base_select_sql, kwargs, limit), kwargs, self.data_type, QueryModeEnum.Many)
        return extra(res) if extra else res

    def select_like(self,
                    likes: Union[list[Like], Like] = {},
                    limit: Limit = None,
                    extra=None) -> list:
        sql: str = self.base_select_sql
        data_dict: dict = {}
        if likes:
            sql += ' where'
            if isinstance(likes, list):
                for like in likes:
                    sql += f'{like} and'
                    data_dict[like.column] = like.value
                sql = sql.rstrip(' and')
            else:
                sql += str(likes)
                data_dict[likes.column] = likes.value
        res, _ = query(sql + str(limit) if limit else sql, data_dict, self.data_type, QueryModeEnum.Many)
        return extra(res) if extra else res

    def update(self, entity, **kwargs):
        if not kwargs:
            return
        sql = f'update {self.name}'
        sql, query_dict = parse_sql_update(sql, entity)
        sql = parse_sql_where(sql, kwargs)
        _, row_id = execute(sql, {**query_dict, **kwargs})
        return row_id

    def delete(self, **kwargs):
        if not kwargs:
            return
        sql = f'delete from {self.name}'
        sql = parse_sql_where(sql, kwargs)
        return execute(sql, kwargs)

    def insert(self, data) -> int:
        sql = f'insert into {self.name}('
        tmp = '('
        data_dict: dict = data.dict()
        for key in data_dict:
            if data_dict[key] or data_dict[key] == 0:
                sql += f'{key}, '
                tmp += f'%({key})s, '
        tmp = tmp.rstrip(', ') + ')'
        sql = sql.rstrip(', ') + ') value' + tmp
        _, row_id = execute(sql, data_dict)
        return row_id

    def insert_batch(self, data: list) -> int:
        sql = f'insert into {self.name}('
        data_dict: dict = data[0].dict()
        columns: list = []
        for key in data_dict:
            if data_dict[key] or data_dict[key] == 0:
                sql += f' {key}, '
                columns.append(key)
        sql = sql.rstrip(', ') + ') values '

        parse_res, query_dict = parse_sql_batch(columns, data)

        _, row_id = execute(sql + parse_res, query_dict)
        return row_id



import functools
import re
from typing import Callable

from pypadb.utils.query_util import execute, parse_sql_batch


def insert(sql: str) -> Callable:
    arg_list = [s.strip() for s in re.findall(r'[(](.*?)[)]', sql)[0].split(',')]
    sql += ' values'

    def deco(fun):
        @functools.wraps(fun)
        def wrapper(data):
            in_sql = sql
            if not isinstance(data, list):
                fun(data)
                template_sql = '('
                for arg in arg_list:
                    template_sql += f'%({arg})s,'
                template_sql = template_sql.rstrip(',') + '),'
                _, row_id = execute(in_sql + template_sql, data.dict())
                return row_id

            for i in data:
                fun(i)
            parse_res, query_dict = parse_sql_batch(arg_list, data)

            _, row_id = execute(in_sql + parse_res, query_dict)
            return row_id

        return wrapper

    return deco



import functools
from types import GenericAlias
from typing import Callable, Any

from pypadb.connection_pool import cursor_type, connection
from pypadb.exception import RequireReturnTypeAnnotation
from pypadb.utils import inspect_util


def select(sql: str, data_type: Any) -> Callable:
    def deco(fun: Callable):
        @functools.wraps(fun)
        def wrapper(*args):
            conn = connection()
            with conn:
                cur = conn.cursor(cursor_type())
                cur.execute(sql, dict(zip([a.name for a in inspect_util.arg_list(fun)], args)))
                func_returns = inspect_util.returns_type(fun)

                if not func_returns:
                    raise RequireReturnTypeAnnotation('require return type annotation')

                first_data = cur.fetchone()
                if first_data is None:
                    return None

                if func_returns == GenericAlias(list, data_type):
                    return [data_type(**first_data), *[data_type(**i) for i in cur.fetchall()]]
                else:
                    return data_type(**first_data)

        return wrapper

    return deco
